# This file updated automatically before PyPI release was created
MAJOR_VERSION = 0
MINOR_VERSION = 5
PATCH_VERSION = "0-beta"
__short_version__ = "{}.{}".format(MAJOR_VERSION, MINOR_VERSION)
__version__ = "{}.{}".format(__short_version__, PATCH_VERSION)
